/*
 * icmd.c
 *
 * This tool takes command line input as an IPMI command.
 * 
 * Author:  Andy Cress  andrew.r.cress@intel.com
 * Copyright (c) 2003 Intel Corporation. 
 *
 * 03/30/04 Andy Cress - created
 * 04/08/04 Andy Cress - display response data, changed usage order
 */
/*M*
Copyright (c) 2004, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

  a.. Redistributions of source code must retain the above copyright notice, 
      this list of conditions and the following disclaimer. 
  b.. Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation 
      and/or other materials provided with the distribution. 
  c.. Neither the name of Intel Corporation nor the names of its contributors 
      may be used to endorse or promote products derived from this software 
      without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *M*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#ifdef WIN32
#include "getopt.h"
#else
#include <getopt.h>
#endif
#include "ipmicmd.h"
 
/*
 * Global variables 
 */
char   fdebug    = 0;
char * progver   = "1.0";
char * progname  = "icmd";

#define PRIVATE_BUS_ID      0x03 // w Sahalee,  the 8574 is on Private Bus 1
#define PERIPHERAL_BUS_ID   0x24 // w mBMC, the 8574 is on the Peripheral Bus
#define MAXLEN 64

char usagemsg[] = "Usage: %s [-x] rsSa netFn/lun cmd bus [data bytes]\n";
static uchar busid = PRIVATE_BUS_ID;

static uchar htoi(uchar *inhex)
{
   // char rghex[16] = "0123456789ABCDEF";
   uchar val;
   uchar c;
   c = inhex[0] & 0x5f;  /* force cap */
   if (c > '9') c += 9;  /* c >= 'A' */
   val = (c & 0x0f) << 4;
   c = inhex[1] & 0x5f;  /* force cap */
   if (c > '9') c += 9;  /* c >= 'A' */
   val += (c & 0x0f);
   return(val);
}

static int send_icmd(uchar *cmd, int len)
{
	uchar inputData[64];
	uchar responseData[64];
	int responseLength;
	uchar completionCode;
	int ret, i;
	uchar netfn, lun;

        /* icmd format: 0=rsSa, 1=netFn/lun, 2=cmd, 3=bus,  4-n=data */
	lun = cmd[1] & 0x03;
	netfn = cmd[1] >> 2;
	/* ipmi_cmd_raw: cmd, netfn, sa, bus, lun, pdata, ... */
	responseLength = sizeof(responseData);
        ret = ipmi_cmd_raw(cmd[2], netfn, cmd[0], cmd[3], lun, &cmd[4], 
			len-4, responseData, &responseLength, &completionCode, 
			fdebug);
	if ((ret != 0) || (completionCode != 0)) {
           printf("ipmi_cmd_raw: ret = %d, ccode %02x\n",
                    ret, completionCode);
	   return(0);
	} else if (responseLength > 0) {
	   /* show the response data */
	   printf("respData[len=%d]: ",responseLength);
	   for (i = 0; i < responseLength; i++)
	      printf("%02x ",responseData[i]);
	   printf("\n");
	}
	return(ret);
}  /*end send_icmd()*/


int
main(int argc, char **argv)
{
   int ret = 0;
   char c;
   uchar devrec[17];
   uchar cmdbuf[MAXLEN];
   int maxlen = MAXLEN;
   int i, istart, cmdlen;

   printf("%s ver %s\n", progname,progver);
   printf("This is a test tool to compose IPMI commands.\n");
   printf("Do not use without knowledge of the IPMI 1.5 specification.\n");

   istart = 1;
   while ( (c = getopt( argc, argv,"x?")) != EOF ) 
      switch(c) {
          case 'x': fdebug = 1; istart++;  break;  /* debug messages */
	  default:
                printf(usagemsg, progname);
                printf(" where -x   shows eXtra debug messages\n"); 
                exit(1);
      }
   if (fdebug) printf("icmd: argc=%d\n",argc);

   /* 
    * Check the Device ID to determine which bus id to use.
    */
   ret = ipmi_getdeviceid(devrec,16);
   if (ret != 0) {
        printf("Cannot do ipmi_getdeviceid, ret = %d\n",ret);
        exit(1);
   } else {
      uchar b, j;
      char *pstr;

      if (fdebug) {
	printf("devid: ");
	for (j = 0; j < 16; j++) printf("%02x ",devrec[j]);
	printf("\n");
      }
      b = devrec[4] & 0x0f;
      j = devrec[4] >> 4;
      if ((devrec[6] == 0x22) && (devrec[7] == 0x03)) { /*NSC mBMC*/
	  pstr = "mBMC";
	  busid = PERIPHERAL_BUS_ID;
      } else {  /* Intel Sahalee = 57 01 */
	  pstr = "BMC";
	  busid = PRIVATE_BUS_ID;
      }
      printf("-- %s version %x.%x, IPMI version %d.%d \n",
             pstr, devrec[2],  devrec[3], b, j);
   }

   for (i = 0; i < istart; i++) {
      argv++; argc--;
   }
   if (argc < maxlen) maxlen = argc;
   if (fdebug) printf("icmd: len=%d, first=%s\n",maxlen,argv[0]);
   for (i = 0; i < maxlen; i++)
   {
      cmdbuf[i] = htoi(argv[i]);
   }
   cmdlen = i;

   if (fdebug) {
        printf("ipmi_cmd: ");
	for (i = 0; i < cmdlen; i++)
	    printf("%02x ",cmdbuf[i]);
        printf("\n");
   }
   if (cmdlen < 4) {
	printf("command length (%d) is too short\n",cmdlen);      
	printf(usagemsg, progname);
   } else {
	ret = send_icmd(cmdbuf,cmdlen); 
	printf("send_icmd ret = %d\n",ret); 
   }
   exit (ret);
}  /* end main()*/

/* end icmd.c */
